function [net, err, to, feat] = DoBackProp_GPU(data, data_ori, targets, net, to, nNodes2, feat1, feat1_ori)
%data: batchSize * frameSize.
%targets: batchSize * nTargets
%net:  the network to be trained, it should be built and initialized
%to: Training options:
% lr: learning rate, wdr: weight decay rate, momentum, !!!!!!!!!!dw{iLayer}: weight change, db{iLayer}: bias change
%err: [difference, # of errors, objective function]

nLayers = length(net);
batchSize = size(data,1);
nClasses = size(targets,2);

%******* Forward activation
feat{1} = data;
assign(1, feat1{1}, feat{1}, ':', [1, 1, nNodes2{1}]);
%feat1{1} = [feat{1}, ones(batchSize,1, GPUsingle)];  % append 1's for biases 
for iLayer = 1:nLayers-1
  %activation
  feat{iLayer+1} = 1./(1+exp( -feat1{iLayer} * net{iLayer}.w) ); %Sigmoid
  assign(1, feat1{iLayer+1}, feat{iLayer+1}, ':', [1, 1, nNodes2{iLayer+1}])
end

%*** Top Layer using soft-max 
act = exp(feat1{nLayers} * net{nLayers}.w);
feat{nLayers+1} = act ./ repmat(sum(act,2),1,nClasses);

%*** Compute top layer errors 
%compute top delta
iLayer = nLayers;
edelta = (targets - feat{iLayer+1}); %softmax with cross entropy. 
%Update top weights  
edelta = edelta / batchSize;
delta2 = edelta * (slice(net{iLayer}.w, [1, 1, (nNodes2{iLayer})], ':'))';

to.dw{iLayer} = to.lr .* (transpose(feat1{iLayer}) * edelta) + to.momentum .* to.dw{iLayer};
net{iLayer}.w = net{iLayer}.w + to.dw{iLayer} - (net{iLayer}.w) * to.wdr;   % plus weight decay

%******* Backpropagation 
for iLayer = nLayers-1:-1:1
%   feat{iLayer+1} = GPUsingle(feat{iLayer+1});

  %signal through activation function
  edelta = delta2 .* feat{iLayer+1} .* (1-feat{iLayer+1}); % batchSize * (nKernels{iLayer}*nOutBands{iLayer})

  %signal through weights
  delta2 = edelta * (slice(net{iLayer}.w, [1, 1, (nNodes2{iLayer})], ':'))';

  %weight change
  to.dw{iLayer} = to.lr .* ( transpose(feat1{iLayer}) * edelta) + to.momentum .* to.dw{iLayer};

  %*** Update weights  
  net{iLayer}.w = net{iLayer}.w + to.dw{iLayer} - (net{iLayer}.w) * to.wdr;

end



%####################compute the training error#######################################
feat_ori{1} = data_ori;
assign(1, feat1_ori{1}, feat_ori{1}, ':', [1, 1, nNodes2{1}]);
for iLayer2 = 1:nLayers-1
  feat_ori{iLayer2+1} = 1./(1+exp( -feat1_ori{iLayer2} * net{iLayer2}.w) ); %batchSize * nf
  assign(1, feat1_ori{iLayer2+1}, feat_ori{iLayer2+1}, ':', [1, 1, nNodes2{iLayer2+1}])
end
act_ori = exp(feat1_ori{nLayers} * net{nLayers}.w);
feat_ori{nLayers+1} = act_ori ./ repmat(sum(act_ori,2),1,nClasses);

edelta_ori = (targets - feat_ori{nLayers+1}); 

err(1) = sum(sum((edelta_ori).^2)) ./ (nClasses);   % mean squre error

targets_CPU = single(targets);
feat_CPU = single(feat_ori{nLayers+1});
[temp, mi1] = max(targets_CPU,[],2);
[temp, mi2] = max(feat_CPU,[],2);

err(2) = sum(mi1 ~= mi2); 
err(3) = -sum(sum(targets_CPU .* log( feat_CPU + 0.1e-15 ) )); % cross entropy 

%###########################################################################
